@@ -10,16 +10,19 @@ use std::ffi::CString;
 use std::panic::{catch_unwind, UnwindSafe};
 use std::process::abort;
 
+pub mod charencoding;
 pub mod cliargsparser;
 pub mod configpaths;
 pub mod fmtstrformatter;
 pub mod fslock;
 pub mod history;
 pub mod human_panic;
+pub mod keycombination;
 pub mod keymap;
 pub mod logger;
 pub mod matchererror;
 pub mod scopemeasure;
+pub mod stflrichtext;
 pub mod utils;
 
 /// Runs a Rust function, and if it panics, calls abort(); otherwise returns what function
@@ -37,6 +40,6 @@ pub unsafe extern "C" fn rs_cstring_free(string: *mut c_char) {
         if string.is_null() {
             return;
         }
-        CString::from_raw(string);
+        drop(CString::from_raw(string));
     })
 }
